"use client"

import { generateImageColors } from "@/actions/images/generateImageColors";
import { Button } from "@/components/ui/button";
import { ImageColor } from "@/generated/prisma";
import { useState, useTransition } from "react";
import { toast } from "sonner";

export default function ImageColors({ colors: initialColors, imageId, fileKey }: { colors: ImageColor[], imageId: string, fileKey: string }) {
  const [colors, setColors] = useState(initialColors);
  const [isPending, startTransition] = useTransition();

  const handleGenerate = () => {
    startTransition(async () => {
      try {
        const newColors = await generateImageColors(imageId, fileKey);
        setColors(newColors);
        toast.success("Colors extracted successfully");
      } catch (err) {
        toast.error("Failed to extract colors");
        console.error(err);
      }
    });
  };

  return (
    <>
      <div className="flex items-center justify-between mb-2">
        <h2 className="font-semibold text-lg">Image Colors</h2>
        <Button size="sm" onClick={handleGenerate} disabled={isPending}>
          {isPending ? "Extracting..." : "Generate Palette"}
        </Button>
      </div >
      {/* <h2 className="font-semibold text-lg mb-2">Extracted Colors</h2>
      <div className="flex flex-wrap gap-2">
        {extractColors.map((color, index) => (
          <div key={index} className="w-10 h-10 rounded" style={{ backgroundColor: color.hex }} title={color.hex}></div>
        ))}
      </div> */}
      <div className="flex flex-wrap gap-2">
        {colors.map((color, index) => (
          <div key={index} className="w-10 h-10 rounded" style={{ backgroundColor: color.hex ?? "#000000" }} title={`Tone ${color.type} - ${color.hex}`}></div>
        ))}
      </div>
    </>
  );
}